require_relative 'player.rb'
require_relative 'die.rb'
require_relative 'treasure_trove.rb'
require_relative 'game_turn.rb'


module StudioGame
  class Game
    def initialize(name) ## initialize the state of the objecct
      @name = name
      @players = []
    end

    def add_player(player)
      @players << player
    end

    def save_to_csv(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end


    def save_high_scores(filename='high_scores.txt')
      File.open(filename, 'w') do |file|
        file.puts "\nKnuckleheads High Scores:"
        @players.sort.each do |player|
        file.puts save_to_csv(player)
        end
      end
    end

    def load_players(filename)
      File.readlines(filename).each do |line|
        add_player(Player.from_csv(line))
      end
    end

    def print_name_and_health(player)
      puts "#{player.name} (#{player.health})"
    end

    def print_name_and_points(player)
      puts "\n#{player.name}\'s point totals:"
      player.each_found_treasure do |treasure|
        # puts "#{treasure}"
        puts "#{treasure.points} total #{treasure.name} points."
        # puts "#{treasure.points} total #{treasure.name} points"

      end
      puts "#{player.points} grand total points"
    end

    def print_stats
      strong_players, weak_players = @players.partition { |player| player.strong? }
      puts "\nGame Statistics:"

      @players.each do |player|
        print_name_and_points(player)
        puts "\n#{total_points} total points from treasures found"
      end



      puts "\n#{strong_players.count} strong players:"
      strong_players.each do | player|
        print_name_and_health(player)
     end

     puts "\n#{weak_players.count} wimpy players:"
     weak_players.each do | player|
      print_name_and_health(player)
     end

     sorted_players = @players.sort

     sorted_players.each do |player|
       puts "#{player.name.ljust(20, '.')}#{player.score}"
     end
   end

    def play(rounds)
      treasure = TreasureTrove::TREASURES

      puts "There are #{treasure.size} treasures to be found:"

      treasure.each do |treasure|
        puts "#{treasure.name} worth #{treasure.points} points"
      end

      puts "\nThere are #{@players.size} players in the game:"
      puts @players

      # puts die.to_s
      1.upto(rounds) do |round|
        puts "\nRound #{round}:"
        @players.each do |player|
        GameTurn.take_turn(player)
        end
      end
    end

    def total_points
      @players.reduce(0) do |sum, player|
        sum += player.points
      end
    end
  end
end
